"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandNames = exports.help = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const constants_1 = require("@pnpm/constants");
const error_1 = __importDefault(require("@pnpm/error"));
const read_project_manifest_1 = require("@pnpm/read-project-manifest");
const store_connection_manager_1 = require("@pnpm/store-connection-manager");
const graceful_fs_1 = __importDefault(require("@pnpm/graceful-fs"));
const supi_1 = require("supi");
const rimraf_1 = __importDefault(require("@zkochan/rimraf"));
const load_json_file_1 = __importDefault(require("load-json-file"));
const render_help_1 = __importDefault(require("render-help"));
const lockfile_1 = require("@yarnpkg/lockfile");
const path_exists_1 = __importDefault(require("path-exists"));
exports.rcOptionsTypes = cliOptionsTypes;
function cliOptionsTypes() {
    return {};
}
exports.cliOptionsTypes = cliOptionsTypes;
function help() {
    return render_help_1.default({
        description: `Generates ${constants_1.WANTED_LOCKFILE} from an npm package-lock.json (or npm-shrinkwrap.json, yarn.lock) file.`,
        url: cli_utils_1.docsUrl('import'),
        usages: [
            'pnpm import',
        ],
    });
}
exports.help = help;
exports.commandNames = ['import'];
async function handler(opts) {
    // Removing existing pnpm lockfile
    // it should not influence the new one
    await rimraf_1.default(path_1.default.join(opts.dir, constants_1.WANTED_LOCKFILE));
    const versionsByPackageNames = {};
    let preferredVersions = {};
    if (await path_exists_1.default(path_1.default.join(opts.dir, 'yarn.lock'))) {
        const yarnPackgeLockFile = await readYarnLockFile(opts.dir);
        getAllVersionsFromYarnLockFile(yarnPackgeLockFile, versionsByPackageNames);
    }
    else if (await path_exists_1.default(path_1.default.join(opts.dir, 'package-lock.json')) ||
        await path_exists_1.default(path_1.default.join(opts.dir, 'npm-shrinkwrap.json'))) {
        const npmPackageLock = await readNpmLockfile(opts.dir);
        getAllVersionsByPackageNames(npmPackageLock, versionsByPackageNames);
    }
    else {
        throw new error_1.default('LOCKFILE_NOT_FOUND', 'No lockfile found');
    }
    preferredVersions = getPreferredVersions(versionsByPackageNames);
    const store = await store_connection_manager_1.createOrConnectStoreController(opts);
    const installOpts = {
        ...opts,
        lockfileOnly: true,
        preferredVersions,
        storeController: store.ctrl,
        storeDir: store.dir,
    };
    await supi_1.install(await read_project_manifest_1.readProjectManifestOnly(opts.dir), installOpts);
}
exports.handler = handler;
async function readYarnLockFile(dir) {
    try {
        const yarnLockFile = await graceful_fs_1.default.readFile(path_1.default.join(dir, 'yarn.lock'), 'utf8');
        const lockJsonFile = await lockfile_1.parse(yarnLockFile);
        if (lockJsonFile.type === 'success') {
            return lockJsonFile.object;
        }
        else {
            throw new error_1.default('GET_YARN_LOCKFILE_ERR', `Failed With ${lockJsonFile.type}`);
        }
    }
    catch (err) {
        if (err['code'] !== 'ENOENT')
            throw err; // eslint-disable-line @typescript-eslint/dot-notation
    }
    throw new error_1.default('YARN_LOCKFILE_NOT_FOUND', 'No yarn.lock found');
}
async function readNpmLockfile(dir) {
    try {
        return await load_json_file_1.default(path_1.default.join(dir, 'package-lock.json'));
    }
    catch (err) {
        if (err['code'] !== 'ENOENT')
            throw err; // eslint-disable-line @typescript-eslint/dot-notation
    }
    try {
        return await load_json_file_1.default(path_1.default.join(dir, 'npm-shrinkwrap.json'));
    }
    catch (err) {
        if (err['code'] !== 'ENOENT')
            throw err; // eslint-disable-line @typescript-eslint/dot-notation
    }
    throw new error_1.default('NPM_LOCKFILE_NOT_FOUND', 'No package-lock.json or npm-shrinkwrap.json found');
}
function getPreferredVersions(versionsByPackageNames) {
    const preferredVersions = {};
    for (const packageName of Object.keys(versionsByPackageNames)) {
        preferredVersions[packageName] = Array.from(versionsByPackageNames[packageName]).reduce((acc, version) => {
            acc[version] = 'version';
            return acc;
        }, {});
    }
    return preferredVersions;
}
function getAllVersionsByPackageNames(npmPackageLock, versionsByPackageNames) {
    if (npmPackageLock.dependencies == null)
        return;
    for (const packageName of Object.keys(npmPackageLock.dependencies)) {
        if (!versionsByPackageNames[packageName]) {
            versionsByPackageNames[packageName] = new Set();
        }
        versionsByPackageNames[packageName].add(npmPackageLock.dependencies[packageName].version);
    }
    for (const packageName of Object.keys(npmPackageLock.dependencies)) {
        getAllVersionsByPackageNames(npmPackageLock.dependencies[packageName], versionsByPackageNames);
    }
}
function getAllVersionsFromYarnLockFile(yarnPackageLock, versionsByPackageNames) {
    for (const packageName of Object.keys(yarnPackageLock)) {
        const pkgName = packageName.substring(0, packageName.lastIndexOf('@'));
        if (!versionsByPackageNames[pkgName]) {
            versionsByPackageNames[pkgName] = new Set();
        }
        versionsByPackageNames[pkgName].add(yarnPackageLock[packageName].version);
    }
}
//# sourceMappingURL=import.js.map